@@ -16,6 +16,8 @@ from django.views.generic.detail import SingleObjectMixin
 
 from braces.views import LoginRequiredMixin, PermissionRequiredMixin
 
+from guardian.mixins import PermissionRequiredMixin as PerObjectPermissionRequiredMixin
+
 from djangocodemirror.views import SampleQuicksaveMixin
 
 from rstview.parser import SourceParser
@@ -25,6 +27,7 @@ from sveedocuments.models import Page, Attachment
 from sveedocuments.forms.page import PageForm, PageEditForm, PageQuickForm
 from sveedocuments.forms.attachment import AttachmentForm
 from sveedocuments.utils.objects import get_instance_children
+from sveedocuments.mixins import PageChangePermissionRequiredMixin
 from sveedocuments.utils.braces_addons import DetailListAppendView, DirectDeleteView, DownloadMixin
 
 class PageIndexView(generic.TemplateView):
@@ -89,7 +92,7 @@ class PageSourceView(PageDetailsView):
             raise Http404
         return HttpResponse(self.get_object().content, content_type="text/plain; charset=utf-8")
 
-class PageCreateView(LoginRequiredMixin, PermissionRequiredMixin, generic.CreateView):
+class PageCreateView(PermissionRequiredMixin, generic.CreateView):
     """
     Form view to create a *Page* document
     """
@@ -154,7 +157,7 @@ class PageTabsContentMixin(object):
         })
         return context
 
-class PageEditView(LoginRequiredMixin, PermissionRequiredMixin, PageTabsContentMixin, generic.UpdateView):
+class PageEditView(PerObjectPermissionRequiredMixin, PageTabsContentMixin, generic.UpdateView):
     """
     Form view to edit a *Page* document
     """
@@ -192,7 +195,7 @@ class PageEditView(LoginRequiredMixin, PermissionRequiredMixin, PageTabsContentM
         return kwargs
 
 
-class PageHistoryView(LoginRequiredMixin, PermissionRequiredMixin, PageTabsContentMixin, generic.DetailView):
+class PageHistoryView(PerObjectPermissionRequiredMixin, PageTabsContentMixin, generic.DetailView):
     """
     *Page* history
     """
@@ -220,46 +223,7 @@ class PageHistoryView(LoginRequiredMixin, PermissionRequiredMixin, PageTabsConte
         return context
 
 
-class PageAttachmentsView(LoginRequiredMixin, PermissionRequiredMixin, PageTabsContentMixin, DetailListAppendView):
-    """
-    Form view to add file attachments to a Page
-    """
-    model = Attachment
-    form_class = AttachmentForm
-    template_name = "sveedocuments/board/page_attachments.html"
-    permission_required = "sveedocuments.change_page"
-    raise_exception = True
-    context_parent_object_name = 'page_instance'
-    
-    def get_parent_object(self):
-        return get_object_or_404(Page, slug=self.kwargs['slug'])
-    
-    def get_queryset(self):
-        return self.parent_object.attachment.all()
-
-    def get_success_url(self):
-        return reverse('sveedocuments:page-attachments', args=[self.parent_object.slug])
-
-    def get_form_kwargs(self):
-        kwargs = super(PageAttachmentsView, self).get_form_kwargs()
-        kwargs.update({'author': self.request.user})
-        return kwargs
-
-
-class PageAttachmentDeleteView(LoginRequiredMixin, PermissionRequiredMixin, DirectDeleteView):
-    """
-    View to delete a *Page* document
-    """
-    model = Attachment
-    permission_required = "sveedocuments.delete_attachment"
-    raise_exception = True
-    memoize_old_object = True
-    _memoized_attr = ['id', 'slug', 'title', 'page']
-
-    def get_success_url(self):
-        return reverse('sveedocuments:page-attachments', args=[self.old_object['page'].slug])
-
-class PageDeleteView(LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView):
+class PageDeleteView(PerObjectPermissionRequiredMixin, generic.DeleteView):
     """
     Form view to delete a *Page* document
     
@@ -287,6 +251,51 @@ class PageDeleteView(LoginRequiredMixin, PermissionRequiredMixin, generic.Delete
         # Then redirect
         return reverse('sveedocuments:page-index')
 
+
+class PageAttachmentsView(PageChangePermissionRequiredMixin, PageTabsContentMixin, DetailListAppendView):
+    """
+    Form view to add file attachments to a Page
+    """
+    model = Attachment
+    form_class = AttachmentForm
+    template_name = "sveedocuments/board/page_attachments.html"
+    raise_exception = True
+    context_parent_object_name = 'page_instance'
+    
+    def get_page_object(self):
+        return self.get_parent_object()
+    
+    def get_parent_object(self):
+        return get_object_or_404(Page, slug=self.kwargs['slug'])
+    
+    def get_queryset(self):
+        return self.parent_object.attachment.all()
+
+    def get_success_url(self):
+        return reverse('sveedocuments:page-attachments', args=[self.parent_object.slug])
+
+    def get_form_kwargs(self):
+        kwargs = super(PageAttachmentsView, self).get_form_kwargs()
+        kwargs.update({'author': self.request.user})
+        return kwargs
+
+
+class PageAttachmentDeleteView(PageChangePermissionRequiredMixin, DirectDeleteView):
+    """
+    View to delete a *Page* document
+    """
+    model = Attachment
+    raise_exception = True
+    memoize_old_object = True
+    _memoized_attr = ['id', 'slug', 'title', 'page']
+    
+    def get_page_object(self):
+        return get_object_or_404(Page, slug=self.kwargs['slug'])
+
+    def get_success_url(self):
+        return reverse('sveedocuments:page-attachments', args=[self.old_object['page'].slug])
+
+
 class PageQuicksaveView(SampleQuicksaveMixin, PageEditView):
     """
     Quicksave view for a *Page* content
